@@ -1721,7 +1721,7 @@ bool SwAligner::backtraceNucleotidesLocalSseI16(
 		int readq = (*qu_)[row];
 		assert_leq(col, origCol);
 		// Get score in this cell
-		bool empty, reportedThru, canMoveThru, branch = false;
+		bool empty = false, reportedThru, canMoveThru, branch = false;
 		int cur = SSEMatrix::H;
 		if(!d.mat_.reset_[row]) {
 			d.mat_.resetRow(row);
