@@ -69,6 +69,7 @@ enum {
 	ARG_QUALS2,                 // --Q2
 	ARG_QSEQ,                   // --qseq
 	ARG_SEED_SUMM,              // --seed-summary
+	ARG_SC_UNMAPPED,            // --soft-clipped-unmapped-tlen
 	ARG_OVERHANG,               // --overhang
 	ARG_NO_CACHE,               // --no-cache
 	ARG_USE_CACHE,              // --cache
@@ -152,6 +153,9 @@ enum {
 	ARG_DESC_FMOPS,             // --desc-fmops
 	ARG_LOG_DP,                 // --log-dp
 	ARG_LOG_DP_OPP,             // --log-dp-opp
+	ARG_XEQ,                    // --xeq
+	ARG_THREAD_CEILING,         // --thread-ceiling
+	ARG_THREAD_PIDDIR,          // --thread-piddir
 	ARG_INTERLEAVED_FASTQ       // --interleaved
 };
 
