@@ -60,6 +60,8 @@ struct PatternParams {
 		bool solexa64_,
 		bool phred64_,
 		bool intQuals_,
+		int trim5_,
+		int trim3_,
 		int sampleLen_,
 		int sampleFreq_,
 		size_t skip_,
@@ -72,6 +74,8 @@ struct PatternParams {
 		solexa64(solexa64_),
 		phred64(phred64_),
 		intQuals(intQuals_),
+		trim5(trim5_),
+		trim3(trim3_),
 		sampleLen(sampleLen_),
 		sampleFreq(sampleFreq_),
 		skip(skip_),
@@ -85,6 +89,8 @@ struct PatternParams {
 	bool solexa64;		  // true -> qualities are on solexa64 scale
 	bool phred64;		  // true -> qualities are on phred64 scale
 	bool intQuals;		  // true -> qualities are space-separated numbers
+	int trim5;            // amt to hard clip from 5' end
+	int trim3;            // amt to hard clip from 3' end
 	int sampleLen;		  // length of sampled reads for FastaContinuous...
 	int sampleFreq;		  // frequency of sampled reads for FastaContinuous...
 	size_t skip;		  // skip the first 'skip' patterns
@@ -188,6 +194,7 @@ class PatternSource {
 public:
 	
 	PatternSource(const PatternParams& p) :
+		pp_(p),
 		readCnt_(0),
 		mutex() { }
 	
@@ -231,11 +238,15 @@ public:
 	
 protected:
 	
-	/// The number of reads read by this PatternSource
+	
+	// Reference to global input-parsing parameters
+	const PatternParams& pp_;
+	
+	// The number of reads read by this PatternSource
 	volatile TReadId readCnt_;
 	
-	/// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
-	/// of this or another other shared object.
+	// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
+	// of this or another other shared object.
 	MUTEX_T mutex;
 };
 
@@ -283,6 +294,10 @@ public:
 	
 private:
 
+	pair<bool, int> nextBatchImpl(
+		PerThreadReadBuf& pt,
+		bool batch_a);
+
 	size_t cur_;			   // index for first read of next batch
 	size_t skip_;			   // # reads to skip
 	bool paired_;			   // whether reads are paired
@@ -367,7 +382,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a) = 0;
+		bool batch_a,
+		unsigned read_idx) = 0;
 
 	/**
 	 * Reset state to handle a fresh file
@@ -404,16 +420,22 @@ protected:
 		return false;
 	}
 	
-	EList<std::string> infiles_;  // filenames for read files
+	EList<std::string> infiles_;	 // filenames for read files
 	EList<bool> errs_;		 // whether we've already printed an error for each file
 	size_t filecur_;		 // index into infiles_ of next file to read
-	FILE *fp_;				 // read file currently being read from
-	gzFile zfp_;
+	FILE *fp_;			 // read file currently being read from
+	gzFile zfp_;			 // compressed version of fp_
 	bool is_open_;			 // whether fp_ is currently open
 	TReadId skip_;			 // number of reads to skip
 	bool first_;			 // parsing first record in first file?
 	char buf_[64*1024];		 // file buffer
 	bool compressed_;
+
+private:
+
+	pair<bool, int> nextBatchImpl(
+		PerThreadReadBuf& pt,
+		bool batch_a);
 };
 
 /**
@@ -450,7 +472,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 
 	/**
 	 * Scan to the next FASTA record (starting with >) and return the first
@@ -488,9 +511,6 @@ public:
 		const PatternParams& p,
 		bool  secondName) :  // whether it's --12/--tab5 or --tab6
 		CFilePatternSource(infiles, p),
-		solQuals_(p.solexa64),
-		phred64Quals_(p.phred64),
-		intQuals_(p.intQuals),
 		secondName_(secondName) { }
 
 	/**
@@ -505,11 +525,9 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 
-	bool solQuals_;		// base qualities are log odds
-	bool phred64Quals_; // base qualities are on -64 scale
-	bool intQuals_;		// base qualities are space-separated strings
 	bool secondName_;	// true if --tab6, false if --tab5
 };
 
@@ -539,10 +557,7 @@ public:
 	QseqPatternSource(
 		const EList<std::string>& infiles,
 		const PatternParams& p) :
-		CFilePatternSource(infiles, p),
-		solQuals_(p.solexa64),
-		phred64Quals_(p.phred64),
-		intQuals_(p.intQuals) { }
+		CFilePatternSource(infiles, p) { }
 
 	/**
 	 * Finalize qseq parsing outside critical section.
@@ -556,11 +571,9 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 
-	bool solQuals_;		// base qualities are log odds
-	bool phred64Quals_; // base qualities are on -64 scale
-	bool intQuals_;		// base qualities are space-separated strings
 	EList<std::string> qualToks_;
 };
 
@@ -579,7 +592,8 @@ public:
 		eat_(length_-1),
 		beginning_(true),
 		bufCur_(0),
-		subReadCnt_(0llu)
+		cur_(0llu),
+		last_(0llu)
 	{
 		assert_gt(freq_, 0);
 		resetForNextFile();
@@ -603,7 +617,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 	
 	/**
 	 * Reset state to be read for the next file.
@@ -613,7 +628,7 @@ protected:
 		name_prefix_buf_.clear();
 		beginning_ = true;
 		bufCur_ = 0;
-		subReadCnt_ = readCnt_;
+		last_ = cur_;
 	}
 
 private:
@@ -629,7 +644,8 @@ private:
 	char name_int_buf_[20]; /// for composing offsets for names
 	size_t bufCur_;		/// buffer cursor; points to where we should
 						/// insert the next character
-	uint64_t subReadCnt_;/// number to subtract from readCnt_ to get
+	uint64_t cur_;
+	uint64_t last_;     /// number to subtract from readCnt_ to get
 						/// the pat id to output (so it resets to 0 for
 						/// each new sequence)
 };
@@ -647,9 +663,6 @@ public:
 		const PatternParams& p, bool interleaved = false) :
 		CFilePatternSource(infiles, p),
 		first_(true),
-		solQuals_(p.solexa64),
-		phred64Quals_(p.phred64),
-		intQuals_(p.intQuals),
 		interleaved_(interleaved) { }
 	
 	virtual void reset() {
@@ -669,7 +682,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 	
 	/**
 	 * Reset state to be ready for the next file.
@@ -679,9 +693,6 @@ protected:
 	}
 
 	bool first_;		// parsing first read in file
-	bool solQuals_;		// base qualities are log odds
-	bool phred64Quals_; // base qualities are on -64 scale
-	bool intQuals_;		// base qualities are space-separated strings
 	bool interleaved_;	// fastq reads are interleaved
 };
 
@@ -716,7 +727,8 @@ protected:
 	 */
 	virtual std::pair<bool, int> nextBatchFromFile(
 		PerThreadReadBuf& pt,
-		bool batch_a);
+		bool batch_a,
+		unsigned read_idx);
 	
 	/**
 	 * Reset state to be ready for the next file.
