@@ -35,7 +35,7 @@ RefRecord fastaRefReadSize(
 	static int lastc = '>'; // last character seen
 
 	// RefRecord params
-	size_t len = 0; // 'len' counts toward total length
+	TIndexOffU len = 0; // 'len' counts toward total length
 	// 'off' counts number of ambiguous characters before first
 	// unambiguous character
 	size_t off = 0;
@@ -78,32 +78,18 @@ RefRecord fastaRefReadSize(
 			// Don't emit a warning, since this might legitimately be
 			// a gap on the end of the final sequence in the file
 			lastc = -1;
-			return RefRecord((uint32_t)off, (uint32_t)len, first);
+			return RefRecord((TIndexOffU)off, (TIndexOffU)len, first);
 		}
 	}
 
 	// Now skip to the first DNA character, counting gap characters
 	// as we go
-	int lc = -1; // last-DNA char variable for color conversion
+	int lc = -1;
 	while(true) {
 		int cat = asc2dnacat[c];
 		if(rparms.nsToAs && cat >= 2) c = 'A';
 		if(cat == 1) {
-			// This is a DNA character
-			if(rparms.color) {
-				if(lc != -1) {
-					// Got two consecutive unambiguous DNAs
-					break; // to read-in loop
-				}
-				// Keep going; we need two consecutive unambiguous DNAs
-				lc = asc2dna[(int)c];
-				// The 'if(off > 0)' takes care of the case where
-				// the reference is entirely unambiguous and we don't
-				// want to incorrectly increment off.
-				if(off > 0) off++;
-			} else {
-				break; // to read-in loop
-			}
+			break; // to read-in loop
 		} else if(cat >= 2) {
 			if(lc != -1 && off == 0) off++;
 			lc = -1;
@@ -116,7 +102,7 @@ RefRecord fastaRefReadSize(
 			}
 			lastc = '>';
 			//return RefRecord(off, 0, false);
-			return RefRecord((uint32_t)off, 0, first);
+			return RefRecord((TIndexOffU)off, 0, first);
 		}
 		c = in.get();
 		if(c == -1) {
@@ -128,17 +114,10 @@ RefRecord fastaRefReadSize(
 			}
 			lastc = -1;
 			//return RefRecord(off, 0, false);
-			return RefRecord((uint32_t)off, 0, first);
+			return RefRecord((TIndexOffU)off, 0, first);
 		}
 	}
-	assert(!rparms.color || (lc != -1));
 	assert_eq(1, asc2dnacat[c]); // C must be unambiguous base
-	if(off > 0 && rparms.color && first) {
-		// Handle the case where the first record has ambiguous
-		// characters but we're in color space; one of those counts is
-		// spurious
-		off--;
-	}
 
 	// in now points just past the first character of a sequence
 	// line, and c holds the first character
@@ -150,24 +129,23 @@ RefRecord fastaRefReadSize(
 		if(cat == 1) {
 			// It's a DNA character
 			assert(cc == 'A' || cc == 'C' || cc == 'G' || cc == 'T');
+			// Check for overflow
+			if((TIndexOffU)(len + 1) < len) {
+				throw RefTooLongException();
+			}
 			// Consume it
 			len++;
 			// Output it
 			if(bpout != NULL) {
-				if(rparms.color) {
-					// output color
-					bpout->write(dinuc2color[asc2dna[(int)c]][lc]);
-				} else if(!rparms.color) {
-					// output nucleotide
-					bpout->write(asc2dna[c]);
-				}
+				// output nucleotide
+				bpout->write(asc2dna[c]);
 			}
 			lc = asc2dna[(int)c];
 		} else if(cat >= 2) {
 			// It's an N or a gap
 			lastc = c;
 			assert(cc != 'A' && cc != 'C' && cc != 'G' && cc != 'T');
-			return RefRecord((uint32_t)off, (uint32_t)len, first);
+			return RefRecord((TIndexOffU)off, (TIndexOffU)len, first);
 		} else {
 			// Not DNA and not a gap, ignore it
 #ifndef NDEBUG
@@ -184,7 +162,7 @@ RefRecord fastaRefReadSize(
 		c = in.get();
 	}
 	lastc = c;
-	return RefRecord((uint32_t)off, (uint32_t)len, first);
+	return RefRecord((TIndexOffU)off, (TIndexOffU)len, first);
 }
 
 #if 0
@@ -273,10 +251,10 @@ fastaRefReadSizes(
 	EList<RefRecord>& recs,
 	const RefReadInParams& rparms,
 	BitpairOutFileBuf* bpout,
-	int& numSeqs)
+	TIndexOff& numSeqs)
 {
-	uint32_t unambigTot = 0;
-	uint32_t bothTot = 0;
+	TIndexOffU unambigTot = 0;
+	size_t bothTot = 0;
 	assert_gt(in.size(), 0);
 	// For each input istream
 	for(size_t i = 0; i < in.size(); i++) {
@@ -284,11 +262,15 @@ fastaRefReadSizes(
 		assert(!in[i]->eof());
 		// For each pattern in this istream
 		while(!in[i]->eof()) {
-			RefRecord rec = fastaRefReadSize(*in[i], rparms, first, bpout);
-			if((unambigTot + rec.len) < unambigTot) {
-				cerr << "Error: Reference sequence has more than 2^32-1 characters!  Please divide the" << endl
-				     << "reference into batches or chunks of about 3.6 billion characters or less each" << endl
-				     << "and index each independently." << endl;
+			RefRecord rec;
+			try {
+				rec = fastaRefReadSize(*in[i], rparms, first, bpout);
+				if((unambigTot + rec.len) < unambigTot) {
+					throw RefTooLongException();
+				}
+			}
+			catch(RefTooLongException& e) {
+				cerr << e.what() << endl;
 				throw 1;
 			}
 			// Add the length of this record.
