@@ -134,7 +134,7 @@ void SamConfig::printAlignedOptFlags(
 	const
 {
 	char buf[1024];
-	assert(summ.best(rd.mate < 2).valid());
+	assert(summ.bestScore(rd.mate < 2).valid());
 	if(print_as_) {
 		// AS:i: Alignment score generated by aligner
 		itoa10<TAlScore>(res.score().score(), buf);
@@ -145,10 +145,10 @@ void SamConfig::printAlignedOptFlags(
 	if(print_xs_) {
 		// XS:i: Suboptimal alignment score
 		AlnScore sco;
-		if(flags.alignedConcordant()) {
-			sco = summ.bestUnchosen(rd.mate < 2);
+		if(flags.partOfPair()) {
+			sco = summ.bestUnchosenPScore(rd.mate < 2);
 		} else {
-			sco = summ.secbest(rd.mate < 2);
+			sco = summ.bestUnchosenUScore();
 		}
 		if(sco.valid()) {
 			itoa10<TAlScore>(sco.score(), buf);
@@ -333,24 +333,24 @@ void SamConfig::printAlignedOptFlags(
 	}
 	if(flags.partOfPair() && print_zp_) {
 		// ZP:i: Score of best concordant paired-end alignment
-		if(summ.bestPaired().valid()) {
+		if(summ.bestCScore().valid()) {
 			WRITE_SEP();
 			o.append("ZP:i:");
-			itoa10<TAlScore>(summ.bestPaired().score(), buf);
+			itoa10<TAlScore>(summ.bestCScore().score(), buf);
 			o.append(buf);
 		}
 		// Zp:i: Score of second-best concordant paired-end alignment
-		if(summ.secbestPaired().valid()) {
+		if(summ.bestUnchosenCScore().valid()) {
 			WRITE_SEP();
 			o.append("Zp:i:");
-			itoa10<TAlScore>(summ.secbestPaired().score(), buf);
+			itoa10<TAlScore>(summ.bestUnchosenCScore().score(), buf);
 			o.append(buf);
 		}
 	}
 	if(print_zu_) {
 		// ZU:i: Score of best unpaired alignment
-		AlnScore best    = (rd.mate <= 1 ? summ.best1()    : summ.best2());
-		AlnScore secbest = (rd.mate <= 1 ? summ.secbest1() : summ.secbest2());
+		AlnScore best    = summ.bestScore(rd.mate <= 1);
+		AlnScore secbest = summ.bestUnchosenPScore(rd.mate <= 1);
 		WRITE_SEP();
 		o.append("ZU:i:");
 		if(best.valid()) {
@@ -521,54 +521,62 @@ void SamConfig::printAlignedOptFlags(
 	}
 	if(print_zt_) {
 		// ZT:Z: Extra features for MAPQ estimation
-		
-		// 1. AS:i for current mate
-		// 2. diff or NA for current mate
-		// 3. Like 1 for opposite mate
-		// 4. Like 2 for opposite mate
-		
 		WRITE_SEP();
 		const bool paired = flags.partOfPair();
 		const TAlScore MN = std::numeric_limits<TAlScore>::min();
 		TAlScore secondBest[2] = {MN, MN};
 		TAlScore thirdBest[2] = {MN, MN};
-		{
-			for (int self = 0; self < (paired ? 2 : 1); self++) {
-				// Second-best
-				AlnScore sco;
-				bool mate1 = rd.mate < 2;
-				if(self > 0) mate1 = !mate1;
-				if(flags.alignedConcordant()) {
-					sco = summ.bestUnchosen(mate1);
-				} else {
-					sco = summ.secbest(mate1);
-				}
-				if(sco.valid()) {
-					secondBest[self] = sco.score();
-				}
+		const int ED_MAX = std::numeric_limits<int>::max();
+		AlnScore best[2] = {res.score(), res.oscore()};
+		TAlScore diffEd[2] = {ED_MAX, ED_MAX};
+		for (int self = 0; self < (paired ? 2 : 1); self++) {
+			// Second-best
+			AlnScore sco;
+			bool mate1 = rd.mate < 2; // first iteration: self
+			if(self > 0) mate1 = !mate1; // second iteration: opposite
+			if(flags.partOfPair()) {
+				sco = summ.bestUnchosenPScore(mate1);
+			} else {
+				sco = summ.bestUnchosenUScore();
+			}
+			if(sco.valid()) {
+				secondBest[self] = sco.score();
+			}
 
-				// Third-best
-				thirdBest[self] = mate1 ? prm.bestLtMinscMate1 : prm.bestLtMinscMate2;
+			// Third-best
+			thirdBest[self] = mate1 ? prm.bestLtMinscMate1 : prm.bestLtMinscMate2;
+			
+			if(flags.partOfPair()) {
+				if(summ.bestUnchosenPDist(mate1).valid()) {
+					diffEd[self] = best[self].basesAligned() - summ.bestUnchosenPDist(mate1).basesAligned();
+				}
+			} else {
+				if(summ.bestUnchosenUDist().valid()) {
+					diffEd[self] = best[self].basesAligned() - summ.bestUnchosenUDist().basesAligned();
+				}
 			}
 		}
-		TAlScore best[2] = {res.score().score(), res.oscore().score()};
 		TAlScore diff[2] = {MN, MN};
 		for(int self = 0; self < 2; self++) {
 			const TAlScore mx = max(secondBest[self], thirdBest[self]);
-			if(best[self] > MN && mx > MN) {
-				diff[self] = best[self] - mx;
+			if(best[self].score() > MN && mx > MN) {
+				diff[self] = best[self].score() - mx;
 			}
 		}
 		TAlScore best_conc = MN, diff_conc = MN;
-		if(paired && summ.bestPaired().valid()) {
-			best_conc = summ.bestPaired().score();
-			if(summ.secbestPaired().valid()) {
-				diff_conc = best_conc - summ.secbestPaired().score();
+		int diffEd_conc = ED_MAX;
+		if(paired && summ.bestCScore().valid()) {
+			best_conc = summ.bestCScore().score();
+			if(summ.bestUnchosenCScore().valid()) {
+				diff_conc = best_conc - summ.bestUnchosenCScore().score();
+			}
+			if(summ.bestUnchosenCDist().valid()) {
+				diffEd_conc = summ.bestCDist().basesAligned() - summ.bestUnchosenCDist().basesAligned();
 			}
 		}
 		o.append("ZT:Z:");
 		// AS:i for current mate
-		itoa10<TAlScore>((int)best[0], buf);
+		itoa10<TAlScore>((int)best[0].score(), buf);
 		o.append(buf);
 		o.append(",");
 		// diff for current mate
@@ -579,9 +587,17 @@ void SamConfig::printAlignedOptFlags(
 			o.append("NA");
 		}
 		o.append(",");
+		// edit distance diff for current mate
+		if(diffEd[0] != ED_MAX) {
+			itoa10<TAlScore>((int)diffEd[0], buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
 		// AS:i for other mate
-		if(best[1] > MN) {
-			itoa10<TAlScore>((int)best[1], buf);
+		if(best[1].score() > MN) {
+			itoa10<TAlScore>((int)best[1].score(), buf);
 			o.append(buf);
 		} else {
 			o.append("NA");
@@ -611,16 +627,42 @@ void SamConfig::printAlignedOptFlags(
 			o.append("NA");
 		}
 		o.append(",");
-		itoa10<TAlScore>((int)(prm.seedsPerNuc * 1000), buf);
+		// Edit distance diff for aligned pairs
+		if(diffEd_conc != ED_MAX) {
+			itoa10<TAlScore>((int)diffEd_conc, buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		// Flags related to seed hits, specific to this mate but not to the
+		// strand aligned to
+		int mate = (rd.mate < 2 ? 0 : 1);
+		o.append(",");
+		itoa10<TAlScore>((int)((prm.seedsPerNucMS[2 * mate] + prm.seedsPerNucMS[2 * mate + 1]) * 1000), buf);
 		o.append(buf);
 		o.append(",");
-		itoa10<TAlScore>((int)(prm.seedPctUnique * 1000), buf);
+		itoa10<TAlScore>((int)((prm.seedPctUniqueMS[2 * mate] + prm.seedPctUniqueMS[2 * mate + 1]) * 1000), buf);
 		o.append(buf);
 		o.append(",");
-		itoa10<TAlScore>((int)(prm.seedPctRep * 1000), buf);
+		itoa10<TAlScore>((int)((prm.seedPctRepMS[2 * mate] + prm.seedPctRepMS[2 * mate + 1]) * 1000), buf);
 		o.append(buf);
 		o.append(",");
-		itoa10<TAlScore>((int)(prm.seedHitAvg + 0.5), buf);
+		itoa10<TAlScore>((int)((prm.seedHitAvgMS[2 * mate] + prm.seedHitAvgMS[2 * mate + 1]) + 0.5f), buf);
+		o.append(buf);
+		// Flags related to seed hits again, but specific both to this mate and
+		// to the strand aligned to
+		int fw = res.fw() ? 0 : 1;
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedsPerNucMS[2 * mate + fw] * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedPctUniqueMS[2 * mate + fw] * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedPctRepMS[2 * mate + fw] * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedHitAvgMS[2 * mate + fw] + 0.5f), buf);
 		o.append(buf);
 	}
 }
