@@ -73,8 +73,6 @@ public:
 		bool print_xss,
 		bool print_yn,
 		bool print_xn,
-		bool print_cs,
-		bool print_cq,
 		bool print_x0,
 		bool print_x1,
 		bool print_xm,
@@ -104,7 +102,8 @@ public:
 		bool print_zm, // FM Index op string for best-first search
 		bool print_zi, // # seed extend loop iters
 		bool print_zp,
-		bool print_zu) :
+		bool print_zu,
+		bool print_zt) :
 		truncQname_(truncQname),
 		omitsec_(omitsec),
 		noUnal_(noUnal),
@@ -120,8 +119,6 @@ public:
 		print_xss_(print_xss),
 		print_yn_(print_yn), // minimum valid score and perfect score
 		print_xn_(print_xn),
-		print_cs_(print_cs),
-		print_cq_(print_cq),
 		print_x0_(print_x0),
 		print_x1_(print_x1),
 		print_xm_(print_xm),
@@ -151,7 +148,8 @@ public:
 		print_zm_(print_zm), // FM Index op string for best-first search
 		print_zi_(print_zi), // # seed extend loop iters
 		print_zp_(print_zp), // # seed extend loop iters
-		print_zu_(print_zu)  // # seed extend loop iters
+		print_zu_(print_zu), // # seed extend loop iters
+		print_zt_(print_zt)  // extra features for MAPQ estimation
 	{
 		assert_eq(refnames_.size(), reflens_.size());
 	}
@@ -234,7 +232,7 @@ public:
 			namelen = 255;
 		}
 		for(size_t i = 0; i < namelen; i++) {
-			if(isspace(name[i])) {
+			if(truncQname_ && isspace(name[i])) {
 				return;
 			}
 			o.append(name[i]);
@@ -283,6 +281,7 @@ public:
 		BTString& o,               // output buffer
 		bool first,                // first opt flag printed is first overall?
 		const Read& rd,            // the read
+		const Read* rdo,           // the opposite read
 		AlnRes& res,               // individual alignment result
 		StackedAln& staln,         // stacked alignment
 		const AlnFlags& flags,     // alignment flags
@@ -345,10 +344,6 @@ protected:
 	bool print_yn_; // YN:i:, Yn:i: minimum valid score and perfect score
 	bool print_xn_; // XN:i: Number of ambiguous bases in the referenece
 
-	// Other optional flags
-	bool print_cs_; // CS:Z: Color read sequence on the original strand
-	bool print_cq_; // CQ:Z: Color read quality on the original strand
-
 	// Following are printed by BWA
 	bool print_x0_; // X0:i: Number of best hits
 	bool print_x1_; // X1:i: Number of sub-optimal best hits
@@ -383,6 +378,7 @@ protected:
 	bool print_zi_; // ZI:i: # extend loop iters
 	bool print_zp_; // ZP:i: Score of best/second-best paired-end alignment
 	bool print_zu_; // ZU:i: Score of best/second-best unpaired alignment
+	bool print_zt_; // ZT:Z: Extra features for MAPQ estimation
 };
 
 #endif /* SAM_H_ */
